/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
#ifndef _AF_H
#define _AF_H
/* Feature test switches */

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE 1
#endif  /* _POSIX_SOURCE */

/* System headers */ 

/* Local Headers */

/* Macros */
#define EMPTYSTR ""
#define STRCMP(a, R, b) (strcmp((a?a:EMPTYSTR), (b?b:EMPTYSTR)) R 0)
#define DATECMP(a, R, b) (datecmp(a, b) R 0)
#define TIMECMP(a, R, b) (timecmp(a, b) R 0)
#define DATETIMECMP(a, R, b) (datetimecmp(a, b) R 0)
#define SIZECMP(a, R, b) (sizecmp(a, b) R 0)

/* special return values to some of the assist functions */
#define SUCCESS		1
#define FAIL		0
#define NULLSTR		(char *)NULL
#define NOSERVER	(Server *)NULL
#define NOQUE		(Que *)NULL
#define NOJOB		(Job *)NULL
#define NOCNODE		(CNode *)NULL
#define EMPTYSETSERVER  (SetServer *)NULL
#define EMPTYSETJOB	(struct SetJobElement *)NULL
#define EMPTYSETQUE	(SetQue *)NULL
#define EMPTYSETCNODE	(SetCNode *)NULL

#define paste2(a, b)	a ## b
#define paste3(a, b, c) a ## b ## c
#define FALSE 0
#define TRUE  1

/* order in sorting */
#define ASC   9
#define DESC  10

#define SIZE_BADVAL    1
#define SIZE_BADSUFFIX 2
/*   NONE */
/* File Scope Variables */
/*  NONE  */
/* External Variables */

struct time_struct {
	int  h;
	int  m;
	int  s;
};

typedef struct time_struct Time;

struct date_struct {
	int  m;
	int  d;
	int  y;		/* full year please */
};

typedef struct date_struct Date;

struct datetime_struct {
	Time t;
	Date d;
};

typedef struct datetime_struct DateTime;

struct size_struct {
	long 	 int num;	/* numeric part */
	unsigned int shift;	/* K=10, M=20, G=30, T=40, P=50 */
	unsigned int units;	/* BYTES=0, WORD=1 */
};
typedef struct size_struct Size;

#define BYTES 0
#define WORDS 1

enum action_type { SYNCRUN, ASYNCRUN, DELETE, RERUN, HOLD, RELEASE, SIGNAL,                                                             MODIFYATTR, MODIFYRES };
typedef enum action_type Action;

enum cpr_type { OP_EQ, OP_NEQ, OP_GT, OP_GE, OP_LT, OP_LE, OP_MAX, OP_MIN };
typedef enum cpr_type Comp;

enum dayofweek_type { SUN, MON, TUE, WED, THU, FRI, SAT };
typedef enum dayofweek_type Dayofweek;

/* Range Types */
struct intRange_struct {
        int lo;
        int hi;
};

typedef struct intRange_struct IntRange;

struct floatRange_struct {
        float lo;
        float hi;
};

typedef struct floatRange_struct FloatRange;

struct dayofweekRange_struct {
        Dayofweek lo;
        Dayofweek hi;
};

typedef struct dayofweekRange_struct DayofweekRange;

struct timeRange_struct {
        Time lo;
        Time hi;
};

typedef struct timeRange_struct TimeRange;

struct dateRange_struct {
        Date lo;
        Date hi;
};

typedef struct dateRange_struct DateRange;

struct datetimeRange_struct {
        DateTime lo;
        DateTime hi;
};

typedef struct datetimeRange_struct DateTimeRange;

struct sizeRange_struct {
        Size lo;
        Size hi;
};

struct IntRes {
	struct IntRes   *nextptr;
	char		*name;
	int		value;
};

struct SizeRes {
	struct SizeRes  *nextptr;
	char		*name;
	Size		value;
};

struct StringRes {
	struct StringRes  *nextptr;
	char		  *name;
	char		  *value;
};

typedef struct sizeRange_struct SizeRange;

/* External Variables */

/* External Functions */
extern void
dynamic_strcpy A_((char **str1_ptr, const char *str2));

extern void
dynamic_strcat A_((char **str1_ptr, const char *str2));

extern int
strtimeToSecs A_((char *times));

extern int
strToInt A_((char *str));

extern double
strToFloat A_((char *str));

extern int
strToDayofweek A_((char *str));

extern Date
strToDate A_((char *str));

extern Time
strToTime A_((char *str));

extern DateTime
strToDateTime A_((char *str));

extern Size
strToSize A_((char *str));

extern IntRange
strToIntRange A_((char *str));

extern FloatRange
strToFloatRange A_((char *str));

extern DayofweekRange
strToDayofweekRange A_((char *str));

extern DateRange
strToDateRange A_((char *str));

extern TimeRange
strToTimeRange A_((char *str));

extern DateTimeRange
strToDateTimeRange A_((char *str));

extern SizeRange
strToSizeRange A_((char *str));

extern DateTime
strsecsToDateTime A_((char *val));

extern int
strToBool A_((char *val));

extern void
sizeToStr A_((Size sizeval, char *cvnbuf));

extern int
datecmp A_((Date d1, Date d2));

extern int
timecmp A_((Time t1, Time t2));

extern int
datetimecmp A_((DateTime dt1, DateTime dt2));

extern DateTime
datetimeGet A_((void));

extern int
datetimeToSecs A_((DateTime dt));

extern int
sizecmp A_((Size a, Size w));

extern Size
sizeAdd A_((Size a, Size w));

extern Size
sizeSub A_((Size a, Size w));

extern Size
sizeMul A_((Size a, Size w));

extern Size
sizeDiv A_((Size a, Size w));

extern Size
sizeUminus A_((Size sz));

extern int
dynamicArraySize A_((void *array));

extern void *
initDynamicArray A_((size_t numElems, size_t elementSize));

extern void *
extendDynamicArray A_((void *array, size_t numElems, size_t elementSize));

extern void
printDynamicArrayTable A_((void));

extern void
freeDynamicArray A_((void *array));

extern void
datePrint A_((Date d));

extern void
timePrint A_((Time t));

extern void
datetimePrint A_((DateTime dt));

extern void
sizePrint A_((Size s, int readable));

extern void
intRangePrint A_((IntRange r));

extern void
floatRangePrint A_((FloatRange r));

extern void
dayofweekPrint A_((Dayofweek dow));

extern void
actPrint A_((Action act));

extern void
cprPrint A_((Comp cpr));

extern void
dayofweekRangePrint A_((DayofweekRange r));

extern void
dateRangePrint A_((DateRange d));

extern void
timeRangePrint A_((TimeRange t));

extern void
datetimeRangePrint A_((DateTimeRange dt));

extern void
sizeRangePrint A_((SizeRange s, int readable));

extern int
sizeRangecmp A_((SizeRange r1, SizeRange r2));

extern int
sizeStrcmp A_((char *a, char *w));

extern int
sizeRangeStrcmp A_((char *str1, char *str2));

extern IntRange
toIntRange A_((int i1, int i2));

extern FloatRange
toFloatRange A_((double f1, double f2));

extern DayofweekRange
toDayofweekRange A_((Dayofweek dow1, Dayofweek dow2));

extern DateRange
toDateRange A_((Date d1, Date d2));

extern TimeRange
toTimeRange A_((Time t1, Time t2));

extern DateTimeRange
toDateTimeRange A_((DateTime dt1, DateTime dt2));

extern SizeRange
toSizeRange A_((Size sz1, Size sz2));

extern char *
strCat A_((char *str1, char *str2));

extern void
varstrAdd A_((void *ptr, int scope, void *pptr));

extern void
varstrRemove A_((void *ptr));

extern void
varstrModScope A_((void *ptr, int newscope));

extern void
varstrModPptr A_((void *ptr, void *newpptr));

extern void
varstrFree A_((void *ptr));

extern void
varstrFreeByScope A_((int scope));

extern void
varstrFreeByPptr A_((void *pptr));

extern void
varstrPrint A_((void));

extern void
mallocTableAdd A_((void *ptr, void *pptr, int scope));

extern void
mallocTablePrint A_((void));

extern void
mallocTableFree A_((void *ptr));

extern void
mallocTableFreeByPptr A_((void *pptr));

extern void
mallocTableFreeByScope A_((int scope, void (*freefunc)()));

extern void
mallocTableModScope A_((void *ptr, int scope));

extern void
mallocTableSafeModScope A_((void *ptr, int scope));

extern int
inMallocTable A_((void *ptr));

extern int
normalizeSize A_((Size *a, Size *b, Size *ta, Size *tb));

/* functions related to IntRes and SizeRes */

extern int
IntResValueGet A_((struct IntRes *head, char *name));

extern struct IntRes *
IntResValuePut A_((struct IntRes *head, char *name, int value, void *pptr));

extern void
IntResListPrint A_((struct IntRes *head, char *descr));

extern void
IntResListFree A_((struct IntRes *head));

extern Size
SizeResValueGet A_((struct SizeRes *head, char *name));

extern struct SizeRes *
SizeResValuePut A_((struct SizeRes *head, char *name, Size value, void *pptr));

extern void
SizeResListPrint A_((struct SizeRes *head, char *descr));

extern void
SizeResListFree A_((struct SizeRes *head));

extern char *
StringResValueGet A_((struct StringRes *head, char *name));

extern struct StringRes *
StringResValuePut A_((struct StringRes *head, char *name, char *value,
                                                                void *pptr));

extern void
StringResListPrint A_((struct StringRes *head, char *descr));

extern void
StringResListFree A_((struct StringRes *head));

#endif		/* _AF_H */
